///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: XGS_AVR_NTSC_TILE_MODE6_V010.h
//
// Original Author: Andre' LaMothe
// 
// Last Modified: 9.2.08
//
// Description: Header file for XGS_AVR_NTSC_TILE_MODE6_V010.s
//
// Overview: 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// watch for multiple inclusions
#ifndef XGS_AVR_NTSC_TILE_MODE6
#define XGS_AVR_NTSC_TILE_MODE6

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// interrupt rates for NTSC/VGA
#define VGA_INTERRUPT_RATE   910
#define NTSC_INTERRUPT_RATE  1817
#define VIDEO_INTERRUPT_RATE (NTSC_INTERRUPT_RATE)

// with the current D/A converter range 0 .. 15, 1 = 62mV
#define NTSC_SYNC           (0x0F)     // 0V  
#define NTSC_BLACK          (0x4F)     // 0.3V (about 30-40 IRE)

#define NTSC_CBURST0        (0x40) 
#define NTSC_CBURST1        (0x41) 
#define NTSC_CBURST2        (0x42) 

#define COLOR_LUMA    (NTSC_CBURST0 + 0x40)

#define NTSC_GREEN    (COLOR_LUMA + 0)        
#define NTSC_YELLOW   (COLOR_LUMA + 2)        
#define NTSC_ORANGE   (COLOR_LUMA + 4)        
#define NTSC_RED      (COLOR_LUMA + 5)        
#define NTSC_VIOLET   (COLOR_LUMA + 9)        
#define NTSC_PURPLE   (COLOR_LUMA + 11)        
#define NTSC_BLUE     (COLOR_LUMA + 14)        

#define NTSC_GRAY0    (NTSC_BLACK + 0x10)        
#define NTSC_GRAY1    (NTSC_BLACK + 0x20)        
#define NTSC_GRAY2    (NTSC_BLACK + 0x30)        
#define NTSC_GRAY3    (NTSC_BLACK + 0x40)        
#define NTSC_GRAY4    (NTSC_BLACK + 0x50)        
#define NTSC_GRAY5    (NTSC_BLACK + 0x60)        
#define NTSC_GRAY6    (NTSC_BLACK + 0x70)        
#define NTSC_GRAY7    (NTSC_BLACK + 0x80)        
#define NTSC_GRAY8    (NTSC_BLACK + 0x90)        
#define NTSC_GRAY9    (NTSC_BLACK + 0xA0)        
#define NTSC_WHITE    (NTSC_BLACK + 0xB0)        


// width and height of screen 
#define NTSC_SCREEN_WIDTH      160
#define NTSC_SCREEN_HEIGHT     (208+1) // +1 needed to initialize the scanline buffer

#define SCREEN_WIDTH      160
#define SCREEN_HEIGHT     (208+1) // +1 needed to initialize the scanline buffer

// controls the number of lines/pixels to shift the display vertically/horizontally
// later assign to global variable, so caller can contro?
#define NTSC_VERTICAL_OFFSET   28
#define NTSC_HORIZONTAL_OFFSET 102

#define VERTICAL_OFFSET   28
#define HORIZONTAL_OFFSET 102

// these indicate the start, end, of the entire video scan and of the active lines, always in terms of real video lines 0
// eg. 0..261 for NTSC, 0..479 for VGA
#define START_VIDEO_SCAN      0
#define START_ACTIVE_SCAN     (VERTICAL_OFFSET)
#define END_ACTIVE_SCAN       (SCREEN_HEIGHT + VERTICAL_OFFSET+1)
#define END_VIDEO_SCAN        261

// simply interogate the video driver's curr_line variable and determine where the scanline is
#define VIDEO_ACTIVE(video_line)          ( ((video_line) >= START_ACTIVE_SCAN && (video_line) <= END_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_INACTIVE(video_line)        ( ((video_line) < START_ACTIVE_SCAN || (video_line) > END_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_TOP_OVERSCAN(video_line)    ( ((video_line) >= 0 && (video_line) < START_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_BOT_OVERSCAN(video_line)    ( ((video_line) > END_ACTIVE_SCAN) ? 1 : 0)

// number of bits each pixel is encoded with
#define NTSC_PIXELS_PER_BYTE   4            
#define PIXELS_PER_BYTE        4            

// number of clocks per pixel (8 in the case of NTSC)
#define CLOCKS_PER_PIXEL       8 

// tile engine defines for physical screen
#define TILE_MAP_WIDTH         18
#define TILE_MAP_HEIGHT        13 

// tile engine bitmap constants
#define TILE_BITMAP_WIDTH      8  // width and height of tile bitmap  
#define TILE_BITMAP_HEIGHT     8
#define TILE_BITMAP_SIZE       64  // size of tile bitmap       

// used to access data structures in ASM easily (if any of these change in "XGS_AVR_GFX_LIB.h" then they must be changed here
#define SPRITE_INDEX_STATE     0
#define SPRITE_INDEX_ATTR      1
#define SPRITE_INDEX_COLOR     2
#define SPRITE_INDEX_HEIGHT    3
#define SPRITE_INDEX_X         4
#define SPRITE_INDEX_Y         5
#define SPRITE_INDEX_BITMAP    6

// sprite attributes
#define SPRITE_ATTR_2X         1  // enable 2X scaling mode

// size of SPRITE type in bytes
#define SPRITE_SIZE            8  

// number of pixels/bytes sprites need to scroll from left edge (basically the max size of a sprite)
#define SPRITE_SCROLL_X        0

// number of sprites driver expects storage for, this can be overidden by re-defining constant in main()
// program, but the driver needs at least this many since it will access the sprite_table[] assuming there
// is at least this storage
#define NUM_SPRITES          5

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// end multiple inclusions 
#endif
