/* db_packet.h - packet transfer routine definitions
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#ifndef __DB_PACKET_H__
#define __DB_PACKET_H__

#include "db_types.h"

/* packet types */
#define PKT_ACK     0x00
#define PKT_NAK     0x01
#define PKT_DEBUG   0x02
#define PKT_GETLINE 0x03
#define PKT_LINE    0x04
#define PKT_BASE    0x10
#define PKT_OPEN    (PKT_BASE + 0x00)
#define PKT_LOAD	(PKT_BASE + 0x01)
#define PKT_RUN     (PKT_BASE + 0x02)
#define PKT_SEQ     (PKT_BASE + 0x03)
#define PKT_DATA    (PKT_SEQ + 0x00)
#define PKT_CLOSE   (PKT_SEQ + 0x01)
#define PKT_LOADC	(PKT_SEQ + 0x02)
#define PKT_LOADI	(PKT_SEQ + 0x03)
#define PKT_START	(PKT_SEQ + 0x04)
#define PKT_NONE    -1

/* block lengths */
#define PKTMAXLEN   128

/* db_sendfile.c */
int SendFile(char *srcname, char *dstname);
int RunFile(char *dstname);

/* db_packet.c */
int PktOpen(char *port);
void PktClose(void);
void PktStartSequence(void);
int PktSendPacket(int type, uint8_t *buf, int len);
void PktSendFrame(int type, uint8_t *buf, int len);
int PktPoll(int timeout);

/* serial i/o definitions */
#define MD_TIMEOUT  -1

/* serial i/o routines */
int md_init(char *port);
void md_done(void);
int md_get(int timeout);
int md_write(uint8_t *buf, int len);

#endif
