#include "db_vm.h"
#include "../common/db_packet.h"

Interpreter interp;

static void DisplaySplashScreen(void);

int main(void)
{
    Interpreter *i = &interp;

    VM_sysinit();

    PktOpen(NULL);
    VM_printf("XGS Basic Runtime %d.%d\n", FILE_VERSION >> 8, FILE_VERSION & 0xff);

    VM_InitImage();
    InitInterpreter(i, TARGET_BITMAP);  // target type doesn't really do anything anymore!

    if (!LoadImageFile(i, "autorun.bai")&& !InitMemoryImage(i))
        DisplaySplashScreen();

    for (;;) {
        switch (i->state) {
        case STATE_STARTING:
        case STATE_RUNNING:
            if (Execute(i, 100) == STATE_IDLE)
                DisplaySplashScreen();
            /* fall through */
        case STATE_LOADING:
        case STATE_PAUSED:
        case STATE_IDLE:
            PktPoll(0);
            break;
        }
    }

    return 0;
}

static void DisplaySplashScreen(void)
{
    char version[20];
    sprintf(version, "%d.%d", FILE_VERSION >> 8, FILE_VERSION & 0xff);
    VM_displayinit();
    VM_displaypos(0, 20);
    VM_displaystr("XGS Basic Runtime ");
    VM_displaystr(version);
    VM_displaypos(0, 40);
    VM_displaystr("ready for download...");
}

void VM_printf(char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    VM_vprintf(fmt, ap);
    va_end(ap);
}

void VM_vprintf(char *fmt, va_list ap)
{
    char buf[80], *p;
    vsnprintf(buf, sizeof(buf), fmt, ap);
    for (p = buf; *p != '\0'; ++p)
        VM_putchar(*p);
}

static char debugBuf[16];
static int debugLen = 0;

static void DebugBufferCh(int ch)
{
    if (debugLen < sizeof(debugBuf)) {
        debugBuf[debugLen++] = ch;
        if (debugLen == sizeof(debugBuf))
            VM_flush();
    }
}

void VM_putchar(int ch)
{
    if (ch == '\n')
        DebugBufferCh('\r');
    DebugBufferCh(ch);
    if (ch == '\n')
        VM_flush();
}

void VM_flush(void)
{
    if (debugLen > 0) {
        PktSendFrame(PKT_DEBUG, (uint8_t *)debugBuf, debugLen);
        debugLen = 0;
    }
}

void VM_getline(void)
{
    VM_flush();
    PktSendFrame(PKT_GETLINE, NULL, 0);
}
