///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: XGS_AVR_GAMEPAD_V010.H
//
// Original Author: Andre' LaMothe
// 
// Last Modified: 8.28.08
//
// Description: Header file for XGS_AVR_GAMEPAD_V010.c
//
// Overview: The game pads are Nintendo Entertainment Systems (NES) based technology. They are slightly different in
// physical representation as well as these controllers have female DB9 rather than the proprietary NES connectors.
// Each controller is the same, so reading each is similar. The controllers use a simple serial encoding scheme where
// the button state is encoded as a stream of 8-bits. To read the game controller state, the caller simply latches
// the data state, then shifts out the button states bit by bit via a clock signal applied to the clock pin.
// The interface is shown below:
//
// I/O pin map for game controllers
// 
// GAMEPAD_DATAOUT_0 | PD4 | Pin 13 | Output
// GAMEPAD_DATAOUT_1 | PD5 | Pin 14 | Output
// GAMEPAD_SH/LDn    | PD6 | Pin 15 | Input
// GAMEPAD_CLK       | PC6 | Pin 26 | Input
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// watch for multiple inclusions
#ifndef XGS_AVR_GAMEPAD_DRV
#define XGS_AVR_GAMEPAD_DRV

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// gamepad I/O declarations
#define GAMEPAD_PORT_DATA_LATCH PORTD
#define GAMEPAD_PORT_CLK        PORTC

// I/O port defines
#define GAMEPAD0            4 // port d
#define GAMEPAD1            5 // port d

#define GAMEPAD_DATA0       4 // port d
#define GAMEPAD_DATA1       5 // port d
#define GAMEPAD_SHnLD       6 // port d
#define GAMEPAD_CLK         7 // port C

// joystick bit mask defines
#define GAMEPAD_RIGHT       0x01
#define GAMEPAD_LEFT        0x02
#define GAMEPAD_DOWN        0x04
#define GAMEPAD_UP          0x08 

#define GAMEPAD_START       0x10
#define GAMEPAD_SELECT      0x20

#define GAMEPAD_ACTION1     0x40
#define GAMEPAD_RED         0x40

#define GAMEPAD_ACTION2     0x80
#define GAMEPAD_YELLOW      0x80

// note the red/yellow button change depending on the color of the controller, so best to use the "action" defines

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Gamepad_Init();
int Gamepad_Read(int gamepadio_pin);

// end multiple inclusions 
#endif
