///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: RPE_FIREDEMO_NTSC_BMP_96_72_4_V010.c
//
// Original Author: Roy Eltham
// 
// Last Modified: 12.29.08
//
// Overview: Fire demo
//
// Controls: gamepad, use <START> button to draw XGS on screen 
// 
// 
// Demo must be linked with following source files (in addition to primary file):
// 
// XGS_AVR_SYSTEM_V010.c         - System variables.
// RPE_NTSC_BMP_96_72_4_V010.s   - ASM NTSC bitmap driver.
// RPE_GFX_LIB_4BIT_V010.c       - Graphics library which includes bitmapped graphics functions, etc.
// XGS_AVR_GAMEPAD_DRV_V010.c    - Gamepad driver.
//
// Note 1: header files for all sources must be in current working directory, so compiler can find them
//
// 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Modifications|Bugs|Additions:
//
// Author Name:
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// define chip type here
#define __AVR_ATmega644__

// define CPU frequency in Mhz here if not defined in Makefile, this is used for conditional compilation, so needs
// to be first, before other XGS includes, also note there is a copy of this constant in XGS_AVR_SYSTEM_V010, so if you 
// change it here, also change it in the header to make certain everthing is sync'ed
#define F_CPU 28636360UL

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <avr/eeprom.h>
#include <avr/io.h>
#include <avr/sleep.h>

#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <compat/twi.h>
#include <avr/pgmspace.h>

// include local XGS API header files
#include "XGS_AVR_SYSTEM_V010.h"
#include "XGS_AVR_GAMEPAD_DRV_V010.h"

// include header file for video driver
#include "RPE_NTSC_BMP_96_72_4_V010.h"
#include "RPE_GFX_LIB_4BIT_V010.h"


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// bitmap for VGA/NTSC
volatile UCHAR vram[SCREEN_WIDTH * SCREEN_HEIGHT / PIXELS_PER_BYTE ];

// pixel mapping for the letters XGS
#define LETTER_TABLE_SIZE ((9*2) + (16*2) + (17*2))
unsigned int letter_table[LETTER_TABLE_SIZE] = {
	30,30,                      38,30, 
	       32,32,        36,32, 
	              34,34, 
	       36,36,        32,36, 
	38,38,                      30,38, // X (9 pairs)

	42,30, 44,30, 46,30, 48,30, 50,30,
	42,32,
	42,34,               48,34, 50,34,
	42,36,                      50,36,
	42,38, 44,38, 46,38, 48,38, 50,38, // G (16 pairs)

	54,30, 56,30, 58,30, 60,30, 62,30,
    54,32,
	54,34, 56,34, 58,34, 60,34, 62,34,
                                62,36,
	54,38, 56,38, 58,38, 60,38, 62,38, // S (17 pairs)
};


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MAIN ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(void)
{
	int gamepad;           // used to read gamepad
	int x, y;
	unsigned char screen_byte;
	unsigned int screen_color0, screen_color1;
	unsigned char* vram_y;
	unsigned char* vram_y_plus_1;
	unsigned char* vram_y_minus_1;

	// initialize graphics variables
	vram_ptr = vram;
	g_screen_width  = SCREEN_WIDTH;
	g_screen_height = SCREEN_HEIGHT;

	// setup palette for fire
	palette_table[0] = NTSC_BLACK;
	palette_table[1] = 0x0E;
	palette_table[2] = 0x25;
	palette_table[3] = 0x65;
	palette_table[4] = 0x85;
	palette_table[5] = 0xB4;
	palette_table[6] = 0xB4;
	palette_table[7] = 0xC3;
	palette_table[8] = 0xC3;
	palette_table[9] = 0xE3;
	palette_table[10] = 0xE3;
	palette_table[11] = 0xF2;
	palette_table[12] = 0xF2;
	palette_table[13] = 0xF2;
	palette_table[14] = 0xF2;
	palette_table[15] = 0xF2;

	// seed random number generator
	srandom(42);

	// initialize the video driver and GFX engine
	GFX_Init( VIDEO_INTERRUPT_RATE );

	// initialize gamepad
	Gamepad_Init();

	// setup processor sleep mode
	set_sleep_mode( SLEEP_MODE_IDLE );
	sleep_enable();

	// clear screen
	GFX_Fill_Screen(0, vram);

	// loop forever
	while(1)
	{
	    // sleep and wait for interrupt
	    sleep_cpu();

	    // only do our processing during vsync.
	    if (curr_line == END_ACTIVE_SCAN+1 )
	    {
	        // check gamepad
	        gamepad = Gamepad_Read(GAMEPAD0);

	        // if it's the <START> button, then draw our letters
	        if (gamepad & GAMEPAD_START)
	        {
				int i;
				for(i = 0; i < LETTER_TABLE_SIZE; i+=2)
				{
					x = letter_table[i];
					y = letter_table[i+1];
					vram[(x >> 1) + (y * (SCREEN_WIDTH >> 1))] = 0xFF;
				} // end for

	        } // end if


			// get pointers to last 2 lines of the vram buffer
			vram_y = vram + (SCREEN_HEIGHT - 2) * (SCREEN_WIDTH >> 1);
			vram_y_plus_1 = vram + (SCREEN_HEIGHT - 1) * (SCREEN_WIDTH >> 1);

			// fuel the fire at the bottom edge of the screen
			for (x = 0; x < SCREEN_WIDTH; x+=2)
			{
				if (RAND_RANGE(0,100) > 33)
				{
					vram_y[x>>1] = 0xFF;
					vram_y_plus_1[x>>1] = 0xFF;
				}
				else
				{
					vram_y[x>>1] = 0x00;
					vram_y_plus_1[x>>1] = 0x00;
				} // end if

			} // end for

			for (y = 1; y < SCREEN_HEIGHT; y++)
			{
				// get pointers to lines of the vram buffer we need
				vram_y = vram + y * (SCREEN_WIDTH >> 1);
				vram_y_plus_1 = vram_y + (SCREEN_WIDTH >> 1);
				vram_y_minus_1 = vram_y - (SCREEN_WIDTH >> 1);
				 
				// we skip the first and last 2 pixels on the edges of the screen to 
				// simplify the pixel processing inside the loop
				// also, we process 2 pixels per loop
				for (x = 2; x < SCREEN_WIDTH-2; x+=2)
				{
					// get the source for 2 pixels
					screen_byte = vram_y[x >> 1];

					// the processing here takes for screen pixels and averages them
					// then writes the results into the pixel just above
					//  (x,y-1) = (x,y) + (x+1,y) + (x-1,y) + (x,y+1) / 4

					// process first pixel 
					screen_color0 = screen_byte & 0x0F;
					screen_color0 += vram_y_plus_1[x >> 1] & 0x0F;
					screen_color0 += (vram_y[(x >> 1) - 1] >> 4) & 0x0F;
					screen_color0 += (screen_byte >> 4) & 0x0F;
					screen_color0 >>= 2;

					// process second pixel
					screen_color1 = (screen_byte >> 4) & 0x0F;
					screen_color1 += (vram_y_plus_1[x >> 1] >> 4) & 0x0F;
					screen_color1 += screen_byte & 0x0F;
					screen_color1 += vram_y[(x >> 1) + 1] & 0x0F;
					screen_color1 >>= 2;

					// write the combined results
					vram_y_minus_1[x >> 1] = screen_color0 | screen_color1 << 4;

					// every other scan line
					if (y & 0x01)
					{
						// fade the "flame" a little

						// if the pixels are not zero then subtract one from them
						screen_byte = (screen_byte & 0x0F) ? screen_byte - 0x01 : screen_byte;
						screen_byte = (screen_byte & 0xF0) ? screen_byte - 0x10 : screen_byte;

						// write the faded pixels
						vram_y[x >> 1] = screen_byte;
					} // end if

				} // end for

			} // end for

	    } // end if

	} // end while

} // end main

